@@ -227,6 +227,8 @@ class Plugin(TrustedCoinPlugin):
     def init_wallet_wizard(self, wizard: 'QENewWalletWizard'):
         wizard.trustedcoin_qhelper = TrustedcoinPluginQObject(self, wizard, None)
         self.extend_wizard(wizard)
+        if wizard.start_viewstate and wizard.start_viewstate.view.startswith('trustedcoin_'):
+            wizard.start_viewstate.params.update({'icon': icon_path('trustedcoin-wizard.png')})
 
     def extend_wizard(self, wizard: 'QENewWalletWizard'):
         super().extend_wizard(wizard)
@@ -357,7 +359,7 @@ class WCChooseSeed(WizardComponent):
         self._valid = True
 
     def apply(self):
-        self.wizard_data['keystore_type'] = self.choice_w.selected_item[0]
+        self.wizard_data['keystore_type'] = self.choice_w.selected_key
 
 
 class WCTerms(WizardComponent):
@@ -402,6 +404,7 @@ class WCShowConfirmOTP(WizardComponent):
     def __init__(self, parent, wizard):
         WizardComponent.__init__(self, parent, wizard, title=_('Authenticator secret'))
         self._otp_verified = False
+        self._is_online_continuation = False
 
         self.new_otp = QWidget()
         new_otp_layout = QVBoxLayout()
@@ -417,8 +420,8 @@ class WCShowConfirmOTP(WizardComponent):
         exist_otp_layout = QVBoxLayout()
         knownlabel = WWLabel(_('This wallet is already registered with TrustedCoin.'))
         exist_otp_layout.addWidget(knownlabel)
-        knownsecretlabel = WWLabel(_('If you still have your OTP secret, then authenticate below to finalize wallet creation'))
-        exist_otp_layout.addWidget(knownsecretlabel)
+        self.knownsecretlabel = WWLabel(_('If you still have your OTP secret, then authenticate below to finalize wallet creation'))
+        exist_otp_layout.addWidget(self.knownsecretlabel)
         self.exist_otp.setLayout(exist_otp_layout)
 
         self.authlabelnew = WWLabel(_('Then, enter your Google Authenticator code:'))
@@ -466,6 +469,10 @@ class WCShowConfirmOTP(WizardComponent):
         self.wizard.trustedcoin_qhelper.otpError.connect(self.on_otp_error)
         self.wizard.trustedcoin_qhelper.remoteKeyError.connect(self.on_remote_key_error)
 
+        self._is_online_continuation = 'seed' not in self.wizard_data
+        if self._is_online_continuation:
+            self.knownsecretlabel.setText(_('Authenticate below to finalize wallet creation'))
+
         self.wizard.trustedcoin_qhelper.createKeystore()
 
     def update(self):
@@ -474,13 +481,15 @@ class WCShowConfirmOTP(WizardComponent):
         self.exist_otp.setVisible(not is_new)
         self.authlabelnew.setVisible(is_new)
         self.authlabelexist.setVisible(not is_new)
-        self.resetlabel.setVisible(not is_new and not self._otp_verified)
-        self.button.setVisible(not is_new and not self._otp_verified)
+        self.authlabelexist.setEnabled(not self._otp_verified)
+        self.otp_e.setEnabled(not self._otp_verified)
+        self.resetlabel.setVisible(not is_new and not self._otp_verified and not self._is_online_continuation)
+        self.button.setVisible(not is_new and not self._otp_verified and not self._is_online_continuation)
 
         if self.wizard.trustedcoin_qhelper.otpSecret:
             self.secretlabel.setText(self.wizard.trustedcoin_qhelper.otpSecret)
             uri = 'otpauth://totp/Electrum 2FA %s?secret=%s&digits=6' % (
-                self.wizard_data['wallet_name'], self.wizard.trustedcoin_qhelper.otpSecret)
+                os.path.basename(self.wizard_data['wallet_name']), self.wizard.trustedcoin_qhelper.otpSecret)
             self.qr.setData(uri)
 
     def on_busy_changed(self):
@@ -520,9 +529,14 @@ class WCShowConfirmOTP(WizardComponent):
     def on_otp_edited(self):
         self.otp_status_l.setVisible(False)
         text = self.otp_e.text()
+        if len(text) > 0:
+            try:
+                otp_int = int(text)
+            except ValueError:
+                return
         if len(text) == 6:
             # verify otp
-            self.wizard.trustedcoin_qhelper.checkOtp(self.wizard.trustedcoin_qhelper.shortId, int(text))
+            self.wizard.trustedcoin_qhelper.checkOtp(self.wizard.trustedcoin_qhelper.shortId, otp_int)
             self.setEnabled(False)
             self.spinner_l.setVisible(True)
             self.spinner.start()
@@ -552,7 +566,7 @@ class WCKeepDisable(WizardComponent):
         self._valid = True
 
     def apply(self):
-        self.wizard_data['trustedcoin_keepordisable'] = self.choice_w.selected_item[0]
+        self.wizard_data['trustedcoin_keepordisable'] = self.choice_w.selected_key
 
 
 class WCContinueOnline(WizardComponent):
